from flask import Flask
from http.server import HTTPServer, BaseHTTPRequestHandler
from pymongo import MongoClient
from pymongo.server_api import ServerApi
from consts import DB_NAME, COLLECTION, MONGO_URI, MONGO_CERT_PATH, SSL_CERT_PATH, SSL_KEY_PATH, HOST, PORT
from map.get_markers import get_markers
from data_structs import Measurement, MarkerTemplate, DateAndTime, Sensor
import ssl
import keyboard
from flask import json
from bson import json_util

app = Flask(__name__)
terminate_server = 0


# Initialise MongoDB connection
def initDatabase():
    try:
        client = MongoClient(MONGO_URI,
                            tls=True,
                            tlsCertificateKeyFile=MONGO_CERT_PATH,
                            server_api=ServerApi('1'))

        db = client[DB_NAME]
        collection = db[COLLECTION]
        print("Connected to MongoDB")
        return client
    except Exception as e:
        print(f"Failed to connect to MongoDB: {e}")


class IceHTTPServer(HTTPServer):
    def __init__(self, server_address, handler_class, client):
        super().__init__(server_address, handler_class)
        self.client = client

    def get_request(self):
        request, client_address = super().get_request()
        return request, client_address

    
# Define custom HTTP class
class IceHTTP(BaseHTTPRequestHandler):
    def __init__(self, request, client_address, server):
        self.client = server.client
        super().__init__(request, client_address, server)
    
    def do_GET(self):
        db = self.client[DB_NAME] 
        col = db[COLLECTION] # Set collection pointer

        # Root path
        if self.path == '/':
            self.send_response(200)
            self.send_header("Content-type", "text/plain")
            self.end_headers()

            self.wfile.write(b"Root path hit!")

        # Update_map endpoint
        elif self.path == '/update_map': # NB: should be POST?
            try:
                # Fetch marker data
                marker_cursor = col.find({})
                marker_data = list(marker_cursor)
                
                # Remove ObjectId field from each document
                for document in marker_data:
                    document.pop('_id', None)

                if marker_data: # Data found in DB, convert from BSON to JSON
                    resp_code = 200 
                    marker_json_list = [json_util.dumps(document) for document in marker_data]
                    marker_json = "[" + ",".join(marker_json_list) + "]" 
                else: # Data not found in DB
                    resp_code = 404 
                    marker_json = '[]' 
            except Exception as e:
                print(f"An error occurred while querying MongoDB: {e}")
                resp_code = 500
                marker_data = []

            # Set headers
            self.send_response(resp_code)
            self.send_header("Content-type", "application/json")
            self.end_headers()

            # Convert marker_data to JSON and write it to the response object
            marker_json = json.dumps(marker_data)
            self.wfile.write(marker_json.encode('utf-8'))

# Listen for pressing of q key to terminate server
def on_key_press(server, event):
    if event.name == 'q':
        print('Terminating server...')
        server.server_close()
        keyboard.unhook_all()
        quit()


# Start a server on port 8443 using self defined HTTP class
if __name__ == "__main__":
    # Initialise database connection
    client = initDatabase()

    try:
        # Load SSL certificate and private key
        ssl_context = ssl.SSLContext(ssl.PROTOCOL_TLS_SERVER)
        ssl_context.load_cert_chain(SSL_CERT_PATH, SSL_KEY_PATH)

        # Create HTTP server with SSL support
        #server = HTTPServer((HOST, PORT), IceHTTP(client))
        server = IceHTTPServer((HOST, PORT), IceHTTP, client)
        server.socket = ssl_context.wrap_socket(server.socket, server_side=True)

        print("Server running on port ", PORT)

        # Register key press event handler
        keyboard.on_press(lambda event: on_key_press(server, event))

        # Run server indefinitely
        server.serve_forever()

    except Exception as e:
        print(f"Server terminated: {PORT}: {e}")
